package com.bytesw.bytebot.controller;


import com.bytesw.bytebot.bean.AgentBean;
import com.bytesw.bytebot.controller.bean.ResponseController;
import com.bytesw.bytebot.service.AgentService;
import com.bytesw.xdf.annotation.ProgramSecurity;
import com.bytesw.xdf.sql.beans.Pagination;
import com.google.gson.GsonBuilder;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import io.swagger.annotations.ApiResponse;
import io.swagger.annotations.ApiResponses;
import lombok.extern.log4j.Log4j2;
import org.apache.commons.lang.exception.ExceptionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

@Log4j2
@RestController
@RequestMapping("/service/agent")
@ProgramSecurity("conversational_agent")
public class AgentController {

    @Autowired
    private AgentService agentService;

    @Autowired
    private GsonBuilder gsonBuilder;

    @PostMapping(value = "/page")
    @PreAuthorize("hasPermission(this, 'view')")
    public ResponseEntity<String> paginationCommercialEquivalence(@RequestBody Pagination<AgentBean> pagination) {
        HttpStatus hs = HttpStatus.OK;

        try {
            agentService.searchByPagination(pagination);
            return new ResponseEntity<>(gsonBuilder.create().toJson(pagination), hs);
        } catch (Exception e) {
            log.error("Error detectado: ", e);

            hs = HttpStatus.INTERNAL_SERVER_ERROR;
            return new ResponseEntity<>(gsonBuilder.create().toJson(ExceptionUtils.getFullStackTrace(e)), hs);
        }
    }


    @GetMapping(value = "/{id}")
    @PreAuthorize("hasPermission(this, 'view')")
    public ResponseEntity<String> getExternalSystemServiceById(@ApiParam(value = "id", required = true) @PathVariable("id") Long id) {
        HttpStatus hs = HttpStatus.OK;

        try {
            return new ResponseEntity<>(gsonBuilder.create().toJson(agentService.getAgent(id)), hs);
        } catch (Exception e) {
            log.error("Error detectado: ", e);

            hs = HttpStatus.INTERNAL_SERVER_ERROR;
            return new ResponseEntity<>(gsonBuilder.create().toJson(ExceptionUtils.getFullStackTrace(e)), hs);
        }
    }

    @PostMapping(value = "/")
    @PreAuthorize("hasPermission(this, 'new')")
    public ResponseEntity<String> createExternalSystem(@RequestBody AgentBean agentBean) {

        HttpStatus hs = HttpStatus.OK;

        try {
            agentService.save(agentBean);
            return new ResponseEntity<>(gsonBuilder.create().toJson(new ResponseController("agent.insert.success", hs.value())), hs);
        } catch (Exception e) {
            log.error("Error detectado: ", e);

            hs = HttpStatus.INTERNAL_SERVER_ERROR;
            return new ResponseEntity<>(gsonBuilder.create().toJson(ExceptionUtils.getFullStackTrace(e)), hs);
        }
    }

    @PostMapping(value = "/{id}")
    @PreAuthorize("hasPermission(this, 'edit')")
    public ResponseEntity<String> updateExternalSystem(@PathVariable("id") Long id,
                                                       @RequestBody AgentBean agentBean) {
        HttpStatus hs = HttpStatus.OK;

        try {
            if (id != null) {
                agentService.save(agentBean);
                return new ResponseEntity<>(gsonBuilder.create().toJson(new ResponseController("agent.update.success", hs.value())), hs);
            } else {
                hs = HttpStatus.UNAUTHORIZED;
                return new ResponseEntity<>(gsonBuilder.create().toJson(new ResponseController("agent.update.error", hs.value())), hs);
            }
        } catch (Exception e) {
            log.error("Error detectado: ", e);

            hs = HttpStatus.INTERNAL_SERVER_ERROR;
            return new ResponseEntity<>(gsonBuilder.create().toJson(ExceptionUtils.getFullStackTrace(e)), hs);
        }
    }

    @DeleteMapping(value = "/{id}")
    @PreAuthorize("hasPermission(this, 'delete')")
    public ResponseEntity<String> deleteExternalSystem(@ApiParam(value = "id", required = true) @PathVariable("id") Long id) {
        HttpStatus hs = HttpStatus.OK;

        try {
            if (id != null) {
                boolean deleteValid = agentService.delete(id);

                if (deleteValid) {
                    return new ResponseEntity<>(gsonBuilder.create().toJson(new ResponseController("agent.delete.success", hs.value())), hs);
                } else {
//                    hs = HttpStatus.FAILED_DEPENDENCY;
                    return new ResponseEntity<>(gsonBuilder.create().toJson(new ResponseController("agent.delete.error.integrity", hs.FAILED_DEPENDENCY.value())), hs);
                }
            } else {
                hs = HttpStatus.UNAUTHORIZED;
                return new ResponseEntity<>(gsonBuilder.create().toJson(new ResponseController("agent.delete.error.unauthtorized", hs.value())), hs);
            }
        } catch (Exception e) {
            log.error("Error detectado: ", e);

            hs = HttpStatus.INTERNAL_SERVER_ERROR;
            return new ResponseEntity<>(gsonBuilder.create().toJson(ExceptionUtils.getFullStackTrace(e)), hs);
        }
    }
}
